;;//-----------------------------LICENSE NOTICE------------------------------------
;;//  This file is part of U.F.O Driver
;;//  Copyright (C) 2019 Hidden Productions
;;//
;;//  This program is free software: you can redistribute it and/or modify
;;//  it under the terms of the GNU General Public License as published by
;;//  the Free Software Foundation, either version 3 of the License, or
;;//  (at your option) any later version.
;;//
;;//  This program is distributed in the hope that it will be useful,
;;//  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;//  GNU General Public License for more details.
;;//
;;//  You should have received a copy of the GNU General Public License
;;//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;//------------------------------------------------------------------------------
;;
;; Game Manager

;; HEADERS
.include "cpctelera.h.s"
.include "render_system.h.s"
.include "entity_manager.h.s"
.include "physics_system.h.s"
.include "input_system.h.s"
.include "ai_control.h.s"
.include "collision_System.h.s"
.include "level_manager.h.s"
.include "game_manager.h.s"
.include "entity_ai.h.s"
.include "map.h.s"
.include "music.h.s"

;; GLOBAL METHODS
.globl cpct_scanKeyboard_f_asm
.globl cpct_isKeyPressed_asm


;;Variable para hacer pressUp una Key
pressUp: .db 0

;;VARIABLES DE CONTROL DE JUEGO
game_state: .db 0

banderaOut: .db 0
colorCogido: .db 0
tocandoRojo: .db 0
tocandoAzul: .db 0
tocandoAmarillo: .db 0
nivelActual: .db 1

banderasActualesColocadas: .db 0
maxBanderasEnNivel: .db 0

x_player: .db 0
y_player: .db 0

life: .db 0

velocity_movement: .db 0

contCallPhysics: .db 0
maxContCallPhysics: .db 1

empieza_musica: .db 1

currentLevelMap: .dw 0

;; Variables para el renderizado de Enemies/Flags/Power ups
;; Por cada nivel habrá distinta cantidad de enemies/flags/power ups
;; Por tanto, necesitamos guardarnos esos datos para luego saber en las
;; funciones de renderizado la cantidad de cada uno a renderizar
enemiesToRender: .db 0
flagsToRender: .db 0
pursuersToRender: .db 0


gameMan_Init::
   ld bc, #state_start_game
   ld  (game_state), bc

   ld a, #1
   ld (nivelActual), a

   ld a, #3
   ld (life), a

   ld hl, #_level_inicio
   ld (currentLevelMap), hl
   call drawTilemap
ret

comprobarNivelCompletado:
   ;; Comprobamos si las banderas colocadas son el mismo número que todas
   ;; las banderas del nivel actual
   ld a, (banderasActualesColocadas)
   ld b, a
   ld a, (maxBanderasEnNivel)
   cp b  ;;SON IGUALEs
   jr nz, _FaltanBanderasPorColocar
      ld a, (nivelActual)
      inc a
      ld (nivelActual), a
      call gameMan_LoadLevel
   _FaltanBanderasPorColocar:

ret

gameMan_Update::
   ;;###CAMBIO DE NIVEL AUTOMATICO
   ld    hl, #Key_M
   ;;DESTROY:A, D, BC, HL
   call cpct_isKeyPressed_asm
   jr   z, M_NotPressed
   M_Pressed:
      ld a, #2
      ld (nivelActual), a
      call gameMan_LoadLevel
   M_NotPressed:


   call gameMan_GameController

   ld    a, (game_state)
   ;; Nos guardamos el estado de nuestro juego
   cp    #state_start_game
   jr    z, _JuegoNoEmpezado

   _JuegoEmpezado:
      ;;MUSICA
        ld a, (empieza_musica) ;;Caraga en a el valor de empieza_musica
        cp #0
        jr z, _noInicia        ;;Si a = 0 la musica se inicia
        call music_init
        ld a, (empieza_musica) ;;Se le vuelve a cargar el valor a a para hacer el xor
        xor #1                 ;;Se cambia el valor de 0 a 1 para no volver a inicializar la musica
        ld (empieza_musica), a ;;Se actualiza el valor de empieza_musica
     _noInicia:

   ld    a, (game_state)
   cp       #state_pause
      ;push af
      ;call cpct_akp_stop_asm
      ;pop af
   call  z, gameMan_pause
   cp       #state_win
   call  z, gameMan_Win
   cp       #state_gameOver
   call  z, gameMan_GameOver
   cp       #state_gaming
   jr    nz, _JuegoNoEmpezado

   _JuegoInGaming:
         ;;PLAY MUSIC
         ;;call music_play
         ;;UPDATE FISICAS
         ld a, (contCallPhysics)
         ld b, a
         ld a, (maxContCallPhysics)
         cp b
         jr nz, _NoCallPhysics

            ;;UPDATE INPUT
            call entityman_getEntityArray_IX
            call entityman_getNumEntities_A
            call inputSys_Update

            ;;UPDATE IA
            call entityman_getEntityArray_IX
            call entityman_getNumEntities_A
            call sys_ai_control_update

            call entityman_getEntityArray_IX
            call entityman_getNumEntities_A
            call physicsSys_Update

            ;;UPDATE COLISIONES
            call entityman_getEntityArray_IX
            call entityman_getNumEntities_A
            call collision_system_Update

            ld a, #0
            ld (contCallPhysics), a

            jr _Continue

         _NoCallPhysics:
            ld a,(contCallPhysics)
            inc a
            ld (contCallPhysics), a
         _Continue:

   _JuegoNoEmpezado:

      _Nothing:

ret

gameMan_Render::
    ;; Comprobamos en el registro A el estado de nuestro juego
    ;; Si el estado no es equivalente al estado de playing, entonces
    ;; no se renderizarán las entidades. En caso contrario, se van a
    ;; renderizar
    ld a, (game_state)
    cp #state_gaming
    ret nz
       call entityman_getEntityArray_IX
       call entityman_getNumEntities_A
       call render_Player
       call render_Enemies
       call render_Pursuers
       call render_Flags

ret

;;METODOS DE CONTROL DE JUEGO
gameMan_GameController:: ;;Para controlar que se pulse pausa (P), startGame (Space), win, gameOver, etc.
   ;;Escaneamos teclado
   call cpct_scanKeyboard_f_asm

   ld    a, (game_state)
   cp    #state_start_game
   jr    z, _CheckStartGame

   _JuegoStart:
      cp #state_gaming
      jr nz, _Continua
         ;;Chequeamos que se pulse la tecla P para pausas
         ld    hl, #Key_L
         call cpct_isKeyPressed_asm
         jr   z, Pe_NotPressed
         Pe_Pressed:
            ld a, #1
            ld (pressUp), a
            jr _Continua
         Pe_NotPressed:
            ld a, (pressUp)
            cp #1
            jr nz, _Continua
               ld a, #0
               ld (pressUp), a
               ld bc, #state_pause
               ld  (game_state), bc
               ld hl, #_level_pause
               call drawTilemap

   _Continua:
      ;;Depende del estado hacemos una cosa u otra
      cp       #state_pause
      jr    z, _ChequearResumeGame

      cp       #state_win
      jr    z, _ChequearResetGame

      cp       #state_gameOver
      jr    z, _ChequearResetGame

      cp       #state_gaming
      jr    z, _NadaMas

      _CheckStartGame:
         ld    hl, #Key_Return
         ;;DESTROY:A, D, BC, HL
         call cpct_isKeyPressed_asm
         jr   z, Space_NotPressed
         Space_Pressed:
            ld bc, #state_gaming
            ld  (game_state), bc
            call gameMan_LoadLevel
         Space_NotPressed:
      ret

      _ChequearResumeGame:
         ld    hl, #Key_L
         ;;DESTROY:A, D, BC, HL
         call cpct_isKeyPressed_asm
         jr   z, P_NotPressed
         ;; Estos dos están comprobando cuando pulsas y levantas el dedo.
         ;; Cuando lo levantes una vez pulsado el botón, entonces se ejecutará
         ;; el código correspondiente
         P_Pressed:
            ld a, #1
            ld (pressUp), a
            jr _ChequearResetGame
         P_NotPressed:
            ld a, (pressUp)
            cp #1
            jr nz, _ChequearResetGame
               ld a, #0
               ld (pressUp), a
               ld bc, #state_gaming
               ld  (game_state), bc
               ld hl, (currentLevelMap)

               call drawTilemap
               call levelMan_drawLifes

      _ChequearResetGame:
         ;;Se pulsa R para resetear la partida
         ld    hl, #Key_R
         ;;DESTROY:A, D, BC, HL
         call cpct_isKeyPressed_asm
         jr   z, R_NotPressed
         R_Pressed:
            call gameMan_ResetGame
         R_NotPressed:
      ret

      _NadaMas:


ret

gameMan_pause::


ret

gameMan_GameOver::
;Muestra pantalla de GameOver.

ret


gameMan_Win::
;Muestra pantalla de juego ganado

ret

gameMan_ResetGame::
;;Resetea el juego
   call gameMan_Init
ret
