;;//-----------------------------LICENSE NOTICE------------------------------------
;;//  This file is part of U.F.O Driver
;;//  Copyright (C) 2019 Hidden Productions
;;//
;;//  This program is free software: you can redistribute it and/or modify
;;//  it under the terms of the GNU General Public License as published by
;;//  the Free Software Foundation, either version 3 of the License, or
;;//  (at your option) any later version.
;;//
;;//  This program is distributed in the hope that it will be useful,
;;//  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;//  GNU General Public License for more details.
;;//
;;//  You should have received a copy of the GNU General Public License
;;//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;//------------------------------------------------------------------------------
;;
;;        Map
;;
.include "cpctelera.h.s"
.include "entity_manager.h.s"
.include "game_manager.h.s"

.globl cpct_etm_setTileset2x4_asm
.globl cpct_etm_drawTileBox2x4_asm
.globl cpct_setPalette_asm
.globl cpct_getScreenPtr_asm
.globl _g_tileset
.globl _g_palette

auxY: .db 0
auxX: .db 0
auxInit: .db 0
;; _g_tileset ---> Es un puntero al array de punteros de cada tile que conforma al tileset
;; _level ---> Es el fichero .tmx con un array donde se han guardado los ID's de cada tile

drawTilemap::
    ;; Guardamos en la pila el puntero hacía nuestro mapa a generar
    push hl
    ;;Creamos nuestro palette de colores
    ld hl, #_g_palette
    ld de, #16
    call cpct_setPalette_asm

    ;; Establecemos el tileset
    ld hl, #_g_tileset
    call cpct_etm_setTileset2x4_asm

    ;; Dibujamos mapa
    ;; Sacamos de la pila el puntero al mapa que queremos generar
    pop hl
    ;; Nos guardamos de nuevo ese puntero en la pila porque la funció
    ;; cpct_etm_drawTileBox2x4_asm lo requiere
    push hl
    ld hl, #0xC000
    push hl
    ld a, #40
    ld b, #0
    ld c, #0
    ld d, #50
    ld e, #40
    call cpct_etm_drawTileBox2x4_asm
ret



mapPlayerCollision_Left::

    ;; En el registro IX tenemos todas las variables de la entidad del Player
    call entityman_getEntityArray_IX

    ;; auxY ---> Variable con la que nos iremos moviendo en el eje "Y" para ir comprobando los pixeles
    ;; hacia abajo del pixel que está a la izquierda del pixel principal desde el que se empieza
    ;; a pintar nuestro "Player"
    ld a, #0
    ld (auxY), a

    ;; auxInit --> Variable que nos servirá de contador para saber cuándo salir exáctamente del loop
    ;; Si auxInit == e_h(ix) (la altura de nuestro sprite) eso significará que ya ha comprobado todos los pixeles
    ;; que están en contacto por la parte izquierda de nuestro sprite y ninguno ha sido del mismo color que el color
    ;; de nuestras paredes, Bright White(0x0B)
    ld a, #0
    ld (auxInit), a

    loop_Left:
        ;; Comparamos auxInit con la altura de nuestro "Player"
        ;; En caso de que sean iguales, eso significa que no se ha encontrado ningún pixel del color de nuestras paredes
        ;; y por tanto podemos indicarle a que se puede mover a la izquierda a nuestro "Player"
        ld a, (auxInit)
        ld b, e_h(ix)
        cp a, b
        jr nz, _Continue
            call moveCharacterLeft
            ret
        _Continue:
            ;; Incrementamos auxInit, auxInit++
            inc a
            ld (auxInit), a

            ;; Recogemos la posición de memoria correspondiente en el registro HL, para comprobar
            ;; posteriormente su color con el color de nuestras paredes
            ;; Tendremos que cargar en el registro C --> X
            ;; Tendremos que cargar en el registro B --> Y
            ;; Tendremos que cargar en el registro DE --> Posición de memoria a partir de la cual se va
            ;; empezar a pintar
            ;; cpct_getScreenPtr_asm devuelve en el registro HL la posición de memoria
            ld de, #0xC000
            ld c, e_x(ix)
            dec c
            ld b, e_y(ix)
            ld a, (auxY)
            add a, b
            ld b, a
            call cpct_getScreenPtr_asm
            ;; Actualizamos la variable auxY (coordenada Y)
            ld a, (auxY)
            inc a
            ld (auxY), a

            ;; Esto nos daría el valor que hay dentro de HL y ese valor lo cargamos en el registro A
            ;; Comparamos el color que hay en el registro A con el color Bright White(0x0B)
            ;; En caso de que los dos sean iguales, se activará el flag Z y por tanto no queremos que siga más comprobando
            ;; y simplemente queremos que salga de la función "mapPlayerCollision_Left"
            ;; En caso contrario, que siga haciendo el loop
            ld a, (hl)
            cp #0xC0 ;;Color rojo
            jr nz, _CompruebaAzul1
                ld a, #1
                ld (tocandoRojo), a
                ld a, #0
                ld (tocandoAzul), a
                ld (tocandoAmarillo), a
                ret
            _CompruebaAzul1:
            cp #0xC3 ;;Color azul
            jr nz, _CompruebaAmarillo1
                ld a, #1
                ld (tocandoAzul), a
                ld a, #0
                ld (tocandoRojo), a
                ld (tocandoAmarillo), a
                ret
            _CompruebaAmarillo1:
            cp #0x03 ;;Color amarillo
            jr nz, _CompruebaBlanco1
                ld a, #1
                ld (tocandoAmarillo), a
                ld a, #0
                ld (tocandoRojo), a
                ld (tocandoAzul), a
                ret
            _CompruebaBlanco1:
            ld a, #0
            ld (tocandoRojo), a
            ld (tocandoAzul), a
            ld (tocandoAmarillo), a
            ld a, (hl)
            cp #12   ;; 11 == 0x0B/12 == 0x0C  __ Por alguna razón me guarda "0C" cuando es blanco en vez de "0B"
            ret z ;; Si el flag Z está activado, nos salimos de la función mapPlayerCollision_Left
    jr nz,loop_Left

ret


mapPlayerCollision_Right::

    ;; En el registro IX tenemos todas las variables de la entidad del Player
    call entityman_getEntityArray_IX

    ;; auxY ---> Variable con la que nos iremos moviendo en el eje "Y" para ir comprobando los pixeles
    ;; hacia abajo del pixel que está a la derecha del pixel principal desde el que se empieza
    ;; a pintar nuestro "Player"
    ld a, #0
    ld (auxY), a

    ;; auxInit --> Variable que nos servirá de contador para saber cuándo salir exáctamente del loop
    ;; Si auxInit == e_h(ix) (la altura de nuestro sprite) eso significará que ya ha comprobado todos los pixeles
    ;; que están en contacto por la parte derecha de nuestro sprite y ninguno ha sido del mismo color que el color
    ;; de nuestras paredes, Bright White(0x0B)
    ld a, #0
    ld (auxInit), a

    loop_Right:
        ;; Comparamos auxInit con la altura de nuestro "Player"
        ;; En caso de que sean iguales, eso significa que no se ha encontrado ningún pixel del color de nuestras paredes
        ;; y por tanto podemos indicarle a que se puede mover a la derecha de nuestro "Player"
        ld a, (auxInit)
        ld b, e_h(ix)
        cp a, b
        jp z, moveCharacterRight

        ret z

        ;; Incrementamos auxInit, auxInit++
        inc a
        ld (auxInit), a

        ;; Recogemos la posición de memoria correspondiente en el registro HL, para comprobar
        ;; posteriormente su color con el color de nuestras paredes
        ;; Tendremos que cargar en el registro C --> X
        ;; Tendremos que cargar en el registro B --> Y
        ;; Tendremos que cargar en el registro DE --> Posición de memoria a partir de la cual se va
        ;; empezar a pintar
        ;; cpct_getScreenPtr_asm devuelve en el registro HL la posición de memoria
        ld de, #0xC000
        ld c, e_x(ix)
        ld a, e_w(ix)
				add a, c
				ld c, a
        ld b, e_y(ix)
        ld a, (auxY)
        add a, b
        ld b, a
        call cpct_getScreenPtr_asm
        ;; Actualizamos la variable auxY (coordenada Y)
        ld a, (auxY)
        inc a
        ld (auxY), a

        ;; Esto nos daría el valor que hay dentro de HL y ese valor lo cargamos en el registro A
        ;; Comparamos el color que hay en el registro A con el color Bright White(0x0B)
        ;; En caso de que los dos sean iguales, se activará el flag Z y por tanto no queremos que siga más comprobando
        ;; y simplemente queremos que salga de la función "mapPlayerCollision_Right"
        ;; En caso contrario, que siga haciendo el loop
        ld a, (hl)
            cp #0xC0 ;;Color rojo
            jr nz, _CompruebaAzul2
                ld a, #1
                ld (tocandoRojo), a
                ld a, #0
                ld (tocandoAzul), a
                ld (tocandoAmarillo), a
                ret
            _CompruebaAzul2:
            cp #0xC3 ;;Color azul
            jr nz, _CompruebaAmarillo2
                ld a, #1
                ld (tocandoAzul), a
                ld a, #0
                ld (tocandoRojo), a
                ld (tocandoAmarillo), a
                ret
            _CompruebaAmarillo2:
            cp #0x03 ;;Color amarillo
            jr nz, _CompruebaBlanco2
                ld a, #1
                ld (tocandoAmarillo), a
                ld a, #0
                ld (tocandoRojo), a
                ld (tocandoAzul), a
                ret
            _CompruebaBlanco2:
            ld a, #0
            ld (tocandoRojo), a
            ld (tocandoAzul), a
            ld (tocandoAmarillo), a
            ld a, (hl)
            cp #12   ;; 11 == 0x0B/12 == 0x0C  __ Por alguna razón me guarda "0C" cuando es blanco en vez de "0B"
            ret z ;; Si el flag Z está activado, nos salimos de la función mapPlayerCollision_Left
    jr nz,loop_Right

ret


mapPlayerCollision_Up::

    ;; En el registro IX tenemos todas las variables de la entidad del Player
    call entityman_getEntityArray_IX

    ;; auxX ---> Variable con la que nos iremos moviendo en el eje "X" para ir comprobando los pixeles
    ;; hacia la derecha del pixel principal desde el que se empieza
    ;; a pintar nuestro "Player"
    ld a, #0
    ld (auxX), a

    ;; auxInit --> Variable que nos servirá de contador para saber cuándo salir exáctamente del loop
    ;; Si auxInit == e_w(ix) (la anchura de nuestro sprite) eso significará que ya ha comprobado todos los pixeles
    ;; que están en contacto por la parte superior de nuestro sprite y ninguno ha sido del mismo color que el color
    ;; de nuestras paredes, Bright White(0x0B)
    ld a, #0
    ld (auxInit), a

    loop_Up:
        ;; Comparamos auxInit con la anchura de nuestro "Player"
        ;; En caso de que sean iguales, eso significa que no se ha encontrado ningún pixel del color de nuestras paredes
        ;; y por tanto podemos indicarle a que se puede mover arriba de  nuestro "Player"
        ld a, (auxInit)
        ld b, e_w(ix)
        cp a, b
        jp z, moveCharacterUp

        ret z

        ;; Incrementamos auxInit, auxInit++
        inc a
        ld (auxInit), a

        ;; Recogemos la posición de memoria correspondiente en el registro HL, para comprobar
        ;; posteriormente su color con el color de nuestras paredes
        ;; Tendremos que cargar en el registro C --> X
        ;; Tendremos que cargar en el registro B --> Y
        ;; Tendremos que cargar en el registro DE --> Posición de memoria a partir de la cual se va
        ;; empezar a pintar
        ;; cpct_getScreenPtr_asm devuelve en el registro HL la posición de memoria
        ld de, #0xC000
        ld c, e_x(ix)
				ld a, (auxX)
				add a, c
				ld c, a
        ld a, e_y(ix)
        dec a
        ld b, a
        call cpct_getScreenPtr_asm
        ;; Actualizamos la variable auxX (coordenada X)
        ld a, (auxX)
        inc a
        ld (auxX), a

        ;; Esto nos daría el valor que hay dentro de HL y ese valor lo cargamos en el registro A
        ;; Comparamos el color que hay en el registro A con el color Bright White(0x0B)
        ;; En caso de que los dos sean iguales, se activará el flag Z y por tanto no queremos que siga más comprobando
        ;; y simplemente queremos que salga de la función "mapPlayerCollision_Up"
        ;; En caso contrario, que siga haciendo el loop
        ld a, (hl)
            cp #0xC0 ;;Color rojo
            jr nz, _CompruebaAzul3
                ld a, #1
                ld (tocandoRojo), a
                ld a, #0
                ld (tocandoAzul), a
                ld (tocandoAmarillo), a
                ret
            _CompruebaAzul3:
            cp #0xC3 ;;Color azul
            jr nz, _CompruebaAmarillo3
                ld a, #1
                ld (tocandoAzul), a
                ld a, #0
                ld (tocandoRojo), a
                ld (tocandoAmarillo), a
                ret
            _CompruebaAmarillo3:
            cp #0x03 ;;Color amarillo
            jr nz, _CompruebaBlanco3
                ld a, #1
                ld (tocandoAmarillo), a
                ld a, #0
                ld (tocandoRojo), a
                ld (tocandoAzul), a
                ret
            _CompruebaBlanco3:
            ld a, #0
            ld (tocandoRojo), a
            ld (tocandoAzul), a
            ld (tocandoAmarillo), a
            ld a, (hl)
            cp #12   ;; 11 == 0x0B/12 == 0x0C  __ Por alguna razón me guarda "0C" cuando es blanco en vez de "0B"
            ret z ;; Si el flag Z está activado, nos salimos de la función mapPlayerCollision_Left
    jr nz,loop_Up

ret



mapPlayerCollision_Down::

    ;; En el registro IX tenemos todas las variables de la entidad del Player
    call entityman_getEntityArray_IX

    ;; auxX ---> Variable con la que nos iremos moviendo en el eje "X" para ir comprobando los pixeles
    ;; en horizontal debajo del pixel principal + su altura desde el que se empieza
    ;; a pintar nuestro "Player"
    ld a, #0
    ld (auxX), a

    ;; auxInit --> Variable que nos servirá de contador para saber cuándo salir exáctamente del loop
    ;; Si auxInit == e_w(ix) (la anchura de nuestro sprite) eso significará que ya ha comprobado todos los pixeles
    ;; que están en contacto por la parte inferior de nuestro sprite y ninguno ha sido del mismo color que el color
    ;; de nuestras paredes, Bright White(0x0B)
    ld a, #0
    ld (auxInit), a

    loop_Down:
        ;; Comparamos auxInit con la anchura de nuestro "Player"
        ;; En caso de que sean iguales, eso significa que no se ha encontrado ningún pixel del color de nuestras paredes
        ;; y por tanto podemos indicarle a que se puede mover abajo de nuestro "Player"
        ld a, (auxInit)
        ld b, e_w(ix)
        cp a, b
        jr z, moveCharacterDown

        ret z

        ;; Incrementamos auxInit, auxInit++
        inc a
        ld (auxInit), a

        ;; Recogemos la posición de memoria correspondiente en el registro HL, para comprobar
        ;; posteriormente su color con el color de nuestras paredes
        ;; Tendremos que cargar en el registro C --> X
        ;; Tendremos que cargar en el registro B --> Y
        ;; Tendremos que cargar en el registro DE --> Posición de memoria a partir de la cual se va
        ;; empezar a pintar
        ;; cpct_getScreenPtr_asm devuelve en el registro HL la posición de memoria
        ld de, #0xC000
        ld c, e_x(ix)
				ld a, (auxX)
				add a, c
				ld c, a
        ld b, e_y(ix)
        ld a, e_h(ix)
				add a, b
        ld b, a
        call cpct_getScreenPtr_asm
        ;; Actualizamos la variable auxX (coordenada X)
        ld a, (auxX)
        inc a
        ld (auxX), a

        ;; Esto nos daría el valor que hay dentro de HL y ese valor lo cargamos en el registro A
        ;; Comparamos el color que hay en el registro A con el color Bright White(0x0B)
        ;; En caso de que los dos sean iguales, se activará el flag Z y por tanto no queremos que siga más comprobando
        ;; y simplemente queremos que salga de la función "mapPlayerCollision_Down"
        ;; En caso contrario, que siga haciendo el loop
        ld a, (hl)
            cp #0xC0 ;;Color rojo
            jr nz, _CompruebaAzul4
                ld a, #1
                ld (tocandoRojo), a
                ld a, #0
                ld (tocandoAzul), a
                ld (tocandoAmarillo), a
                ret
            _CompruebaAzul4:
            cp #0xC3 ;;Color azul
            jr nz, _CompruebaAmarillo4
                ld a, #1
                ld (tocandoAzul), a
                ld a, #0
                ld (tocandoRojo), a
                ld (tocandoAmarillo), a
                ret
            _CompruebaAmarillo4:
            cp #0x03 ;;Color amarillo
            jr nz, _CompruebaBlanco4
                ld a, #1
                ld (tocandoAmarillo), a
                ld a, #0
                ld (tocandoRojo), a
                ld (tocandoAzul), a
                ret
            _CompruebaBlanco4:
            ld a, #0
            ld (tocandoRojo), a
            ld (tocandoAzul), a
            ld (tocandoAmarillo), a
            ld a, (hl)
            cp #12   ;; 11 == 0x0B/12 == 0x0C  __ Por alguna razón me guarda "0C" cuando es blanco en vez de "0B"
            ret z ;; Si el flag Z está activado, nos salimos de la función mapPlayerCollision_Left
    jr nz,loop_Down

ret




moveCharacterLeft:
  ;;ld a, (velocity_movement)
	;;neg
	;;ld 	e_vx(ix), a  ;;Fuerza a la izquierda (negativa porque es A)
	ld 	e_vx(ix), #-1
ret

moveCharacterRight:
	;;ld a, (velocity_movement)
	;;ld e_vx(ix), a   ;; Fuerza hacía la derecha
	ld 	e_vx(ix), #1
ret

moveCharacterUp:
	;;ld a, (velocity_movement)
	;;neg
	;;ld e_vy(ix), a
	ld 	e_vy(ix), #-2	  ;;Fuerza hacia arriba
ret

moveCharacterDown:
	;;ld a, (velocity_movement)
	;;ld e_vy(ix), a
	ld 	e_vy(ix), #2	 ;;Fuerza hacia abajo
ret
