;;//-----------------------------LICENSE NOTICE------------------------------------
;;//  This file is part of U.F.O Driver
;;//  Copyright (C) 2019 Hidden Productions
;;//
;;//  This program is free software: you can redistribute it and/or modify
;;//  it under the terms of the GNU General Public License as published by
;;//  the Free Software Foundation, either version 3 of the License, or
;;//  (at your option) any later version.
;;//
;;//  This program is distributed in the hope that it will be useful,
;;//  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;//  GNU General Public License for more details.
;;//
;;//  You should have received a copy of the GNU General Public License
;;//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;//------------------------------------------------------------------------------
;;
;; Render System

;; HEADERS
.include "entity_manager.h.s"
.include "game_manager.h.s"

;; GLOBAL METHODS
.globl cpct_getScreenPtr_asm
.globl cpct_drawSolidBox_asm
.globl cpct_drawSprite_asm


;; VARIABLES GLOBALES PARA LOS SPRITES DE NUESTRAS ENTIDADES
.globl _g_player

.globl _g_enemy_01
.globl _g_enemy_02

.globl _g_flag_01
.globl _g_flag_02
.globl _g_flag_03

.globl _g_pursuer_01
.globl _g_pursuer_02
.globl _g_pursuer_03

;; Variables que se usan como contador en los métodos de renderizado
;; de los flags y los enemies para saber si hemos llegado al número deseado
;; de enemigos/flags a renderizar para un determinado nivel
contRenderEnemies: .db 0
contRenderFlags: .db 0
contRenderPursuers: .db 0

whatEnemyrender: .db 0

;; Tenemos que renderizar por separado las entidades(player/enemies/flags)
;; ya que los parámetros que le pasaremos a la función serán distintos para
;; cada entidad

;; Cargaremos en el registro B el número de enemigos que queramos en un determinado nivel
;; Cargaremos en el registro C el número de flags que queramos en un determinado nivel

render_Player::
 ;; Borramos el rastro del player
 ld  e, e_lst_vmeml(ix)
 ld  d, e_lst_vmemh(ix)  ;; Load last positon
 xor a     ;; xor A sería equivalente a realizar un ld a, #0
 ld  c, e_w(ix)    ;; Entity_Width
 ld  b, e_h(ix)    ;; Entity_Height
 call cpct_drawSolidBox_asm

 ;; Calculamos el puntero de pantalla desde el que se empieza a pintar
 ;; nuestro player
 ld    de, #0xC000    ;;Start of memory
 ld     c, e_x(ix)          ;; x [0-79]
 ld     b, e_y(ix)      ;;y [0-199]
 call cpct_getScreenPtr_asm

 ;; Nos guardamos última posición donde ha estado el jugador para así borrar
 ;; su rastro al pintarlo de nuevo
 ld  e_lst_vmeml(ix), l
 ld  e_lst_vmemh(ix), h

 ;; Función que nos dibujará nuestro sprite del player
 ;; Datos necesarios:
 ;; HL ---> Pointer al array de C generado para el player
 ;; DE ---> Posición de memoria (página) en la que se va a pintar
 ;; C ---> Anchura del jugador en BYTES (no en pixels)
 ;; B ---> Altura del jugador en PIXELS
 ex de, hl
 ld hl, #_g_player
 ld c, e_w(ix)
 ld b, e_h(ix)
 call cpct_drawSprite_asm

ret


render_Enemies::

 ;; Sabemos de que tenemos en el registro B los enemies que aparecerán
 ;; Iremos comprobando si llega a ese número
 ;; Primero tendremos que avanzar ix para que empiece directamente desde
 ;; nuestra primera entidad enemy y no desde nuestro player

 ;; Iremos comprobando contEnemies con el número de enemies (los que tengamos en el registro B)
 _renEnemiesLoop:
  ld a, (contRenderEnemies)
  ld b, a
  ld a, (enemiesToRender)
  cp b
  jr z, _dontRenderEnemies

  ;; Con esto estamos cambiando a la entidad que apunta el registro IX
  ld bc, #sizeof_e
  add ix, bc

  ;; Incrementamos contEnemies
  ld a, (contRenderEnemies)
  inc a
  ld (contRenderEnemies), a

  ;; Borramos el rastro de los enemies
  ld  e, e_lst_vmeml(ix)
  ld  d, e_lst_vmemh(ix)  ;; Load last positon
  xor a     ;; xor A sería equivalente a realizar un ld a, #0
  ld  c, e_w(ix)    ;; Entity_Width
  ld  b, e_h(ix)    ;; Entity_Height
  call cpct_drawSolidBox_asm

  ;; Calculamos el puntero de pantalla desde el que se empieza a pintar
  ;; nuestro enemy
  ld    de, #0xC000    ;;Start of memory
  ld     c, e_x(ix)          ;; x [0-79]
  ld     b, e_y(ix)      ;;y [0-199]
  call cpct_getScreenPtr_asm

  ;; Nos guardamos última posición donde ha estado el enemy para así borrar
  ;; su rastro al pintarlo de nuevo
  ld  e_lst_vmeml(ix), l
  ld  e_lst_vmemh(ix), h

  ;; Función que nos dibujará nuestro sprite del enemy
  ;; Datos necesarios:
  ;; HL ---> Pointer al array de C generado para el player
  ;; DE ---> Posición de memoria (página) en la que se va a pintar
  ;; C ---> Anchura del jugador en BYTES (no en pixels)
  ;; B ---> Altura del jugador en PIXELS
  ex de, hl

    ld a, (contRenderEnemies)
    cp #1
    jr z, _EsImpar
    cp #3
    jr z, _EsImpar
    cp #5
    jr z, _EsImpar
    cp #7
    jr z, _EsImpar

      ld a, (whatEnemyrender)
      cp #0 
      jr nz, _QuizaSea1
        ld hl, #_g_enemy_01
        ld a, #1
        ld (whatEnemyrender), a
        jr _Fuera
      _QuizaSea1:
      cp #1
      jr nz, _Fuera
        ld hl, #_g_enemy_02
        ld a, #0
        ld (whatEnemyrender), a
        jr _Fuera

    _EsImpar:
      ld hl, #_g_enemy_01
  _Fuera:

  ld c, e_w(ix)
  ld b, e_h(ix)
  call cpct_drawSprite_asm

 jr _renEnemiesLoop

 _dontRenderEnemies:
  ld a, #0
  ld (contRenderEnemies), a
ret



render_Flags::

 ;; Sabemos de que tenemos en el registro B los enemies que aparecerán
 ;; Iremos comprobando si llega a ese número
 ;; Primero tendremos que avanzar ix para que empiece directamente desde
 ;; nuestra primera entidad enemy y no desde nuestro player

 ;; Iremos comprobando contEnemies con el número de enemies (los que tengamos en el registro B)
 _renFlagsLoop:
  ld a, (contRenderFlags)
  ld b, a
  ld a, (flagsToRender)
  cp b
  jr z, _dontRenderFlags

  ;; Con esto estamos cambiando a la entidad que apunta el registro IX
  ld bc, #sizeof_e
  add ix, bc

  ;; Incrementamos contEnemies
  ld a, (contRenderFlags)
  inc a
  ld (contRenderFlags), a

  ;; Borramos el rastro de los enemies
  ld  e, e_lst_vmeml(ix)
  ld  d, e_lst_vmemh(ix)  ;; Load last positon
  xor a     ;; xor A sería equivalente a realizar un ld a, #0
  ld  c, e_w(ix)    ;; Entity_Width
  ld  b, e_h(ix)    ;; Entity_Height
  call cpct_drawSolidBox_asm

  ;; Calculamos el puntero de pantalla desde el que se empieza a pintar
  ;; nuestro enemy
  ld    de, #0xC000    ;;Start of memory
  ld     c, e_x(ix)          ;; x [0-79]
  ld     b, e_y(ix)      ;;y [0-199]
  call cpct_getScreenPtr_asm

  ;; Nos guardamos última posición donde ha estado el enemy para así borrar
  ;; su rastro al pintarlo de nuevo
  ld  e_lst_vmeml(ix), l
  ld  e_lst_vmemh(ix), h


  ;; Comprobar tipo de flag a dibujar
  ex de, hl

  ld a, e_colorTipo(ix)
  cp #e_color1
  jr nz, _ComprobarColor2
    ld hl, #_g_flag_01
    jr _DibujarFlag
  _ComprobarColor2:
  cp #e_color2
  jr nz, _ComprobarColor3
    ld hl, #_g_flag_03
    jr _DibujarFlag
  _ComprobarColor3:
  cp #e_color3
  jr nz, _DibujarFlag
    ld hl, #_g_flag_02
  
  _DibujarFlag:
  ;; Función que nos dibujará nuestro sprite del enemy
  ;; Datos necesarios:
  ;; HL ---> Pointer al array de C generado para el player
  ;; DE ---> Posición de memoria (página) en la que se va a pintar
  ;; C ---> Anchura del jugador en BYTES (no en pixels)
  ;; B ---> Altura del jugador en PIXELS
  
  
  ld c, e_w(ix)
  ld b, e_h(ix)
  call cpct_drawSprite_asm

 jr _renFlagsLoop

 _dontRenderFlags:
  ld a, #0
  ld (contRenderFlags), a
ret




render_Pursuers::

 ;; Sabemos de que tenemos en el registro B los enemies que aparecerán
 ;; Iremos comprobando si llega a ese número
 ;; Primero tendremos que avanzar ix para que empiece directamente desde
 ;; nuestra primera entidad enemy y no desde nuestro player

 ;; Iremos comprobando contEnemies con el número de enemies (los que tengamos en el registro B)
 _renPursuersLoop:
  ld a, (contRenderPursuers)
  ld b, a
  ld a, (pursuersToRender)
  cp b
  jr z, _dontRenderPursuers

  ;; Con esto estamos cambiando a la entidad que apunta el registro IX
  ld bc, #sizeof_e
  add ix, bc

  ;; Incrementamos contEnemies
  ld a, (contRenderPursuers)
  inc a
  ld (contRenderPursuers), a

  ;; Borramos el rastro de los enemies
  ld  e, e_lst_vmeml(ix)
  ld  d, e_lst_vmemh(ix)  ;; Load last positon
  xor a     ;; xor A sería equivalente a realizar un ld a, #0
  ld  c, e_w(ix)    ;; Entity_Width
  ld  b, e_h(ix)    ;; Entity_Height
  call cpct_drawSolidBox_asm

  ;; Calculamos el puntero de pantalla desde el que se empieza a pintar
  ;; nuestro enemy
  ld    de, #0xC000    ;;Start of memory
  ld     c, e_x(ix)          ;; x [0-79]
  ld     b, e_y(ix)      ;;y [0-199]
  call cpct_getScreenPtr_asm

  ;; Nos guardamos última posición donde ha estado el enemy para así borrar
  ;; su rastro al pintarlo de nuevo
  ld  e_lst_vmeml(ix), l
  ld  e_lst_vmemh(ix), h


  ;; Comprobar tipo de flag a dibujar
  ex de, hl

  ld a, e_colorTipo(ix)
  cp #e_color1
  jr nz, _ComprobarColorPursuer2
    ld hl, #_g_pursuer_01
    jr _DibujarPursuer
  _ComprobarColorPursuer2:
  cp #e_color2
  jr nz, _ComprobarColorPursuer3
    ld hl, #_g_pursuer_03
    jr _DibujarPursuer
  _ComprobarColorPursuer3:
  cp #e_color3
  jr nz, _DibujarPursuer
    ld hl, #_g_pursuer_02
  
  _DibujarPursuer:
  ;; Función que nos dibujará nuestro sprite del enemy
  ;; Datos necesarios:
  ;; HL ---> Pointer al array de C generado para el player
  ;; DE ---> Posición de memoria (página) en la que se va a pintar
  ;; C ---> Anchura del jugador en BYTES (no en pixels)
  ;; B ---> Altura del jugador en PIXELS
  
  
  ld c, e_w(ix)
  ld b, e_h(ix)
  call cpct_drawSprite_asm

 jr _renPursuersLoop

 _dontRenderPursuers:
  ld a, #0
  ld (contRenderPursuers), a
ret